@@ -396,7 +396,8 @@ import net.morilib.util.Stack2;
 		List<Datum> no2 = new ArrayList<Datum>();
 
 		retPoint:
-		for(int i = 0;; i++) {
+//		for(int i = 0;; i++) {
+		while(true) {
 			List<Datum> arg = new ArrayList<Datum>();
 
 			//for(int j = 0; j < lst2.size(); j++) {
@@ -870,10 +871,15 @@ import net.morilib.util.Stack2;
 
 				switch(c.getOp()) {
 				case PUSH:
-					// 命令を分けるかも
-					if(c.getDatum() instanceof ClosureClass) {
+					Datum aaa = c.getDatum();
+
+					if(c.getDatum() instanceof LispLazySubrSyn) {
+						aaa = ((LispLazySubrSyn)aaa).setup();
+					}
+
+					if(aaa instanceof ClosureClass) {
 						Closure ncl = new Closure(
-								(ClosureClass)c.getDatum(), env);
+								(ClosureClass)aaa, env);
 //						Environment e2 = env.copyExceptRoot();
 //						Closure ncl = new Closure(
 //								(ClosureClass)c.getDatum(),
@@ -881,8 +887,8 @@ import net.morilib.util.Stack2;
 
 						//System.out.println(ncl.getParameterList());
 						m.dataStk.push(ncl);
-					} else if(c.getDatum() instanceof UserSyntax) {
-						UserSyntax syn = (UserSyntax)c.getDatum();
+					} else if(aaa instanceof UserSyntax) {
+						UserSyntax syn = (UserSyntax)aaa;
 
 						if(syn.getExecuteEnv() != null) {
 							// do nothing
@@ -899,7 +905,7 @@ import net.morilib.util.Stack2;
 					//	p.setEnvironment(env);
 					//	m.dataStk.push(p);
 					} else {
-						m.dataStk.push(c.getDatum());
+						m.dataStk.push(aaa);
 					}
 					addr++;
 					break;
@@ -1253,6 +1259,10 @@ import net.morilib.util.Stack2;
 						d51 = env.findDatum(c.getDatum());
 					}
 
+					if(d51 instanceof LispLazySubrSyn) {
+						d51 = ((LispLazySubrSyn)d51).setup();
+					}
+
 					if(d51 == null) {
 						Datum zx = c.getDatum();
 
