@@ -30,7 +30,7 @@ import net.morilib.util.io.CharUnreadBuffer;
  *
  * @author MORIGUCHI, Yuichiro 2009
  */
-/*pacakge*/ final class IntLispUtils {
+/*package*/ final class IntLispUtils {
 
 	//
 	private IntLispUtils() {
@@ -256,6 +256,9 @@ import net.morilib.util.io.CharUnreadBuffer;
 		_log.finer(str + (dt / 1000) + "." + msec + "sec");
 	}
 
+	//
+	static boolean loadEager = false;
+
 	/*package*/ static void loadJavaSubr(
 			Environment env, Symbol k, String v) {
 		//try {
@@ -275,7 +278,11 @@ import net.morilib.util.io.CharUnreadBuffer;
 		//} catch (IllegalAccessException e) {
 		//	throw new RuntimeException(e);
 		//}
-		env.bindDatum(k, getJavaSubr(k, v));
+		if(loadEager) {
+			env.bindDatum(k, getJavaSubr(k, v));
+		} else {
+			env.bindDatum(k, new LispLazySubrSyn(env, k, v));
+		}
 	}
 
 	/*package*/ static Datum getJavaSubr(Symbol k, String v) {
