@@ -18,8 +18,11 @@ package net.morilib.lisp;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 
+import net.morilib.lang.Decimal64;
 import net.morilib.lang.number.Integer2;
 import net.morilib.lang.number.Rational;
+import net.morilib.lisp.math.InvalidOrdinalNumberException;
+import net.morilib.lisp.math.LispOrdinalNumber;
 import net.morilib.util.Inclementor;
 
 /**
@@ -28,8 +31,11 @@ import net.morilib.util.Inclementor;
  *
  * @author MORIGUCHI, Yuichiro 2009
  */
-public final class LispBigInt
-extends LispInteger implements JavaObjective {
+public final class LispBigInt extends LispInteger
+implements JavaObjective, java.io.Serializable {
+
+	//
+	private static final long serialVersionUID = 3485888605431065348L;
 
 	//
 	private BigInteger value;
@@ -47,13 +53,7 @@ extends LispInteger implements JavaObjective {
 	 * @see net.morilib.lisp.LispNumber#add(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber add(LispNumber x) {
-		if(x instanceof LispComplex) {
-			LispComplex c = (LispComplex)x;
-
-			return LispComplex.newComplex(
-					getRealDouble() + c.getRealDouble(),
-					c.getImagDouble());
-		} else if(x instanceof LispInteger) {
+		if(x instanceof LispInteger) {
 			return LispInteger.valueOf(value.add(x.getBigInteger()));
 		} else if(x instanceof LispRational) {
 			LispRational r = (LispRational)x;
@@ -62,11 +62,9 @@ extends LispInteger implements JavaObjective {
 					r.getDenominator()).add(r.getNumerator());
 
 			return LispRational.newRational(nn, nd);
-		} else if(x instanceof LispDouble) {
-			return new LispDouble(
-					value.doubleValue() + ((LispDouble)x).doubleValue());
+		} else {
+			return super.add(x);
 		}
-		throw new IllegalArgumentException(x.toString());
 	}
 
 	/*
@@ -74,19 +72,7 @@ extends LispInteger implements JavaObjective {
 	 * @see net.morilib.lisp.LispNumber#div(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber div(LispNumber x) {
-		if(x instanceof LispComplex) {
-			double xr = ((LispComplex)x).getRealDouble();
-			double xi = ((LispComplex)x).getImagDouble();
-			double n  = getRealDouble();
-
-			if(xr == 0.0) {
-				return LispComplex.newComplex(0, -n / xi);
-			} else {
-				return LispComplex.newComplex(
-						(n * xr)    / (xr * xr + xi * xi),
-						(-(n * xi)) / (xr * xr + xi * xi));
-			}
-		} else if(x instanceof LispInteger) {
+		if(x instanceof LispInteger) {
 			return LispRational.newRational(value, x.getBigInteger());
 		} else if(x instanceof LispRational) {
 			LispRational r = (LispRational)x;
@@ -94,11 +80,9 @@ extends LispInteger implements JavaObjective {
 			BigInteger nn = value.multiply(r.getDenominator());
 
 			return LispRational.newRational(nn, nd);
-		} else if(x instanceof LispDouble) {
-			return new LispDouble(
-					value.doubleValue() / ((LispDouble)x).doubleValue());
+		} else {
+			return super.div(x);
 		}
-		throw new IllegalArgumentException(x.toString());
 	}
 
 	/*
@@ -106,18 +90,7 @@ extends LispInteger implements JavaObjective {
 	 * @see net.morilib.lisp.LispNumber#mul(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber mul(LispNumber x) {
-		if(x instanceof LispComplex) {
-			LispComplex c = (LispComplex)x;
-
-			if(c.getRealDouble() == 0.0) {
-				return LispComplex.newComplex(
-						0, getRealDouble() * c.getImagDouble());
-			} else {
-				return LispComplex.newComplex(
-						getRealDouble() * c.getRealDouble(),
-						getRealDouble() * c.getImagDouble());
-			}
-		} else if(x instanceof LispInteger) {
+		if(x instanceof LispInteger) {
 			return LispInteger.valueOf(
 					value.multiply(x.getBigInteger()));
 		} else if(x instanceof LispRational) {
@@ -126,11 +99,11 @@ extends LispInteger implements JavaObjective {
 			BigInteger nn = value.multiply(r.getNumerator());
 
 			return LispRational.newRational(nn, nd);
-		} else if(x instanceof LispDouble) {
-			return new LispDouble(
-					value.doubleValue() * ((LispDouble)x).doubleValue());
+		} else if(x instanceof LispIrrational) {
+			return x.mul(this);
+		} else {
+			return super.mul(x);
 		}
-		throw new IllegalArgumentException(x.toString());
 	}
 
 	/*
@@ -138,13 +111,7 @@ extends LispInteger implements JavaObjective {
 	 * @see net.morilib.lisp.LispNumber#sub(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber sub(LispNumber x) {
-		if(x instanceof LispComplex) {
-			LispComplex c = (LispComplex)x;
-
-			return LispComplex.newComplex(
-					getRealDouble() - c.getRealDouble(),
-					-c.getImagDouble());
-		} else if(x instanceof LispInteger) {
+		if(x instanceof LispInteger) {
 			return LispInteger.valueOf(
 					value.subtract(x.getBigInteger()));
 		} else if(x instanceof LispRational) {
@@ -154,18 +121,16 @@ extends LispInteger implements JavaObjective {
 					r.getDenominator()).subtract(r.getNumerator());
 
 			return LispRational.newRational(nn, nd);
-		} else if(x instanceof LispDouble) {
-			return new LispDouble(
-					value.doubleValue() - ((LispDouble)x).doubleValue());
+		} else {
+			return super.sub(x);
 		}
-		throw new IllegalArgumentException(x.toString());
 	}
 
 	/*
 	 * (non-Javadoc)
 	 * @see net.morilib.lisp.LispNumber#uminus()
 	 */
-	public LispNumber uminus() {
+	public LispInteger uminus() {
 		return new LispBigInt(value.negate());
 	}
 
@@ -174,9 +139,7 @@ extends LispInteger implements JavaObjective {
 	 * @see net.morilib.lisp.LispNumber#isEqualTo(net.morilib.lisp.LispNumber)
 	 */
 	public boolean isEqualTo(LispNumber x) {
-		if(x instanceof LispComplex) {
-			return false;
-		} else if(x instanceof LispSmallInt) {
+		if(x instanceof LispSmallInt) {
 			return false;
 		} else if(x instanceof LispBigInt) {
 			return value.equals(x.getBigInteger());
@@ -185,7 +148,7 @@ extends LispInteger implements JavaObjective {
 		} else if(x instanceof LispDouble) {
 			return value.doubleValue() == ((LispDouble)x).doubleValue();
 		}
-		throw new IllegalArgumentException(x.toString());
+		return false;
 	}
 
 	/*
@@ -203,6 +166,9 @@ extends LispInteger implements JavaObjective {
 			return n1.compareTo(n2) < 0;
 		} else if(x instanceof LispDouble) {
 			return value.doubleValue() < x.getRealDouble();
+		} else if(x instanceof LispAlternatingSeriesNumber) {
+			return ((LispAlternatingSeriesNumber)x).value.compareTo(
+					getRational()) > 0;
 		}
 		throw new IllegalArgumentException(x.toString());
 	}
@@ -222,6 +188,9 @@ extends LispInteger implements JavaObjective {
 			return n1.compareTo(n2) > 0;
 		} else if(x instanceof LispDouble) {
 			return value.doubleValue() > x.getRealDouble();
+		} else if(x instanceof LispAlternatingSeriesNumber) {
+			return ((LispAlternatingSeriesNumber)x).value.compareTo(
+					getRational()) < 0;
 		}
 		throw new IllegalArgumentException(x.toString());
 	}
@@ -321,6 +290,14 @@ extends LispInteger implements JavaObjective {
 		return value.doubleValue();
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispNumber#getRealDecimal64()
+	 */
+	@Override
+	public long getRealDecimal64() {
+		return Decimal64.toDecimal(new BigDecimal(value));
+	}
+
 	/*
 	 * (non-Javadoc)
 	 * @see net.morilib.util.Inclementor#equalIncliment(net.morilib.util.Inclementor)
@@ -487,6 +464,39 @@ extends LispInteger implements JavaObjective {
 		return value;
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.math.LispOrdinalNumber#inclement()
+	 */
+	public LispOrdinalNumber inclement() {
+		if(value.signum() < 0) {
+			throw new InvalidOrdinalNumberException();
+		}
+		return LispInteger.valueOf(value.add(BigInteger.ONE));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispReal#remainder(net.morilib.lisp.LispReal)
+	 */
+	@Override
+	public LispReal remainder(LispReal r) {
+		BigInteger n, d;
+
+		if(!r.isExact()) {
+			return new LispDouble(Math.IEEEremainder(
+					value.doubleValue(), r.doubleValue()));
+		} else if(r instanceof LispInteger) {
+			n = value.remainder(r.getBigInteger());
+			return LispInteger.valueOf(n);
+		} else if(r instanceof LispRational) {
+			d = r.getDenominator();
+			n = value.remainder(r.getNumerator());
+			return LispRational.newRational(n, d);
+		} else {
+			return new LispDouble(Math.IEEEremainder(
+					value.doubleValue(), r.doubleValue()));
+		}
+	}
+
 	/*
 	 * (non-Javadoc)
 	 * @see java.lang.Object#equals(java.lang.Object)
