@@ -18,6 +18,8 @@ package net.morilib.lisp;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 
+import net.morilib.lang.Decimal32;
+import net.morilib.lang.Decimal64;
 import net.morilib.lang.number.AbstractNumericalField;
 import net.morilib.lang.number.Double2;
 import net.morilib.lang.number.Integer2;
@@ -31,7 +33,7 @@ import net.morilib.lisp.sos.LispType;
  *
  * @author MORIGUCHI, Yuichiro 2009
  */
-public final class LispDouble extends LispReal
+public class LispDouble extends LispInexactReal
 implements JavaObjective, java.io.Serializable {
 
 	//
@@ -109,7 +111,7 @@ implements JavaObjective, java.io.Serializable {
 	public static final LispDouble NaN = new LispDouble(Double.NaN);
 
 	//
-	private double number;
+	/*package*/ double number;
 
 	/**
 	 * 
@@ -172,13 +174,33 @@ implements JavaObjective, java.io.Serializable {
 	 * @see net.morilib.lisp.LispNumber#add(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber add(LispNumber x) {
-		if(x instanceof LispComplex) {
-			LispComplex c = (LispComplex)x;
-
-			return LispComplex.newComplex(
-					number + c.getRealDouble(), c.getImagDouble());
+		if(x instanceof LispRangedDouble) {
+			return new LispRangedDouble(number + x.getRealDouble(),
+					((LispRangedDouble)x).error);
+		} else if(x instanceof LispHalf) {
+			return new LispDouble(number + x.getRealDouble());
+		} else if(x instanceof LispFloat) {
+			return new LispDouble(number + x.getRealDouble());
+		} else if(x instanceof LispDecimal32) {
+			return new LispDecimal32(Decimal32.add(
+					Decimal32.toDecimal(number),
+					x.getRealDecimal32()));
+		} else if(x instanceof LispDecimal64) {
+			return new LispDecimal64(Decimal64.add(
+					Decimal64.toDecimal(number),
+					x.getRealDecimal64()));
 		} else if(x instanceof LispReal) {
 			return new LispDouble(number + x.getRealDouble());
+		} else if(x instanceof LispComplex) {
+			LispComplex c = (LispComplex)x;
+
+			if(x == LispComplex.INFINITY)  return LispComplex.INFINITY;
+			return LispComplex.newComplex(
+					add(c.getReal()), c.getImag());
+		} else if(x instanceof LispQuaternion) {
+			return LispQuaternion.add(this, (LispQuaternion)x);
+		} else if(x instanceof LispOctonion) {
+			return LispOctonion.add(this, (LispOctonion)x);
 		}
 		throw new IllegalArgumentException(x.toString());
 	}
@@ -188,13 +210,33 @@ implements JavaObjective, java.io.Serializable {
 	 * @see net.morilib.lisp.LispNumber#sub(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber sub(LispNumber x) {
-		if(x instanceof LispComplex) {
-			LispComplex c = (LispComplex)x;
-
-			return LispComplex.newComplex(
-					number - c.getRealDouble(), -c.getImagDouble());
+		if(x instanceof LispRangedDouble) {
+			return new LispRangedDouble(number - x.getRealDouble(),
+					((LispRangedDouble)x).error);
+		} else if(x instanceof LispHalf) {
+			return new LispDouble(number - x.getRealDouble());
+		} else if(x instanceof LispFloat) {
+			return new LispDouble(number - x.getRealDouble());
+		} else if(x instanceof LispDecimal32) {
+			return new LispDecimal32(Decimal32.subtract(
+					Decimal32.toDecimal(number),
+					x.getRealDecimal32()));
+		} else if(x instanceof LispDecimal64) {
+			return new LispDecimal64(Decimal64.subtract(
+					Decimal64.toDecimal(number),
+					x.getRealDecimal64()));
 		} else if(x instanceof LispReal) {
 			return new LispDouble(number - x.getRealDouble());
+		} else if(x instanceof LispComplex) {
+			LispComplex c = (LispComplex)x;
+
+			if(x == LispComplex.INFINITY)  return LispComplex.INFINITY;
+			return LispComplex.newComplex(
+					subtract(c.getReal()), c.getImag().uminus());
+		} else if(x instanceof LispQuaternion) {
+			return LispQuaternion.sub(this, (LispQuaternion)x);
+		} else if(x instanceof LispOctonion) {
+			return LispOctonion.sub(this, (LispOctonion)x);
 		}
 		throw new IllegalArgumentException(x.toString());
 	}
@@ -204,19 +246,34 @@ implements JavaObjective, java.io.Serializable {
 	 * @see net.morilib.lisp.LispNumber#mul(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber mul(LispNumber x) {
-		if(x instanceof LispComplex) {
-			LispComplex c = (LispComplex)x;
-
-			if(c.getRealDouble() == 0.0) {
-				return LispComplex.newComplex(
-						0, number * c.getImagDouble());
-			} else {
-				return LispComplex.newComplex(
-						number * c.getRealDouble(),
-						number * c.getImagDouble());
-			}
+		if(x instanceof LispRangedDouble) {
+			return LispRangedDouble.valueOf(
+					number * x.getRealDouble(),
+					number * ((LispRangedDouble)x).error);
+		} else if(x instanceof LispHalf) {
+			return new LispDouble(number * x.getRealDouble());
+		} else if(x instanceof LispFloat) {
+			return new LispDouble(number * x.getRealDouble());
+		} else if(x instanceof LispDecimal32) {
+			return new LispDecimal32(Decimal32.multiply(
+					Decimal32.toDecimal(number),
+					x.getRealDecimal32()));
+		} else if(x instanceof LispDecimal64) {
+			return new LispDecimal64(Decimal64.multiply(
+					Decimal64.toDecimal(number),
+					x.getRealDecimal64()));
 		} else if(x instanceof LispReal) {
 			return new LispDouble(number * x.getRealDouble());
+		} else if(x instanceof LispComplex) {
+			LispComplex c = (LispComplex)x;
+
+			if(x == LispComplex.INFINITY)  return LispComplex.INFINITY;
+			return LispComplex.newComplex(
+					multiply(c.getReal()), multiply(c.getImag()));
+		} else if(x instanceof LispQuaternion) {
+			return LispQuaternion.mul(this, (LispQuaternion)x);
+		} else if(x instanceof LispOctonion) {
+			return LispOctonion.mul(this, (LispOctonion)x);
 		}
 		throw new IllegalArgumentException(x.toString());
 	}
@@ -226,19 +283,47 @@ implements JavaObjective, java.io.Serializable {
 	 * @see net.morilib.lisp.LispNumber#div(net.morilib.lisp.LispNumber)
 	 */
 	public LispNumber div(LispNumber x) {
-		if(x instanceof LispComplex) {
-			double xr = ((LispComplex)x).getRealDouble();
-			double xi = ((LispComplex)x).getImagDouble();
+		double y1, y2;
+		LispRangedDouble x0;
 
-			if(xr == 0.0) {
-				return LispComplex.newComplex(0, -number / xi);
-			} else {
-				return LispComplex.newComplex(
-						(number * xr)    / (xr * xr + xi * xi),
-						(-(number * xi)) / (xr * xr + xi * xi));
+		if(x instanceof LispRangedDouble) {
+			x0 = (LispRangedDouble)x;
+			if(x0.isZeroIncluded()) {
+				return LispDouble.NaN;
 			}
+			y1 = number / x0.getRealDouble() + number * x0.error;
+			y2 = number / x0.getRealDouble() - number * x0.error;
+			return LispRangedDouble.valueOf(
+					(y1 + y2) / 2.0, Math.abs(y1 - y2) / 2.0);
+		} else if(x instanceof LispHalf) {
+			return new LispDouble(number / x.getRealDouble());
+		} else if(x instanceof LispFloat) {
+			return new LispDouble(number / x.getRealDouble());
+		} else if(x instanceof LispDecimal32) {
+			return new LispDecimal32(Decimal32.divide(
+					Decimal32.toDecimal(number),
+					x.getRealDecimal32()));
+		} else if(x instanceof LispDecimal64) {
+			return new LispDecimal64(Decimal64.divide(
+					Decimal64.toDecimal(number),
+					x.getRealDecimal64()));
 		} else if(x instanceof LispReal) {
 			return new LispDouble(number / x.getRealDouble());
+		} else if(x instanceof LispComplex) {
+			LispReal xr = x.getReal();
+			LispReal xi = x.getImag();
+			LispReal xn = xr.multiply(xr).add(xi.multiply(xi));
+
+			if(x == LispComplex.INFINITY) {
+				return isExact() ? LispInteger.ZERO : LispDouble.ZERO;
+			}
+			return LispComplex.newComplex(
+					multiply(xr).divide(xn),
+					multiply(xi).uminus().divide(xn));
+		} else if(x instanceof LispQuaternion) {
+			return LispQuaternion.div(this, (LispQuaternion)x);
+		} else if(x instanceof LispOctonion) {
+			return LispOctonion.div(this, (LispOctonion)x);
 		}
 		throw new IllegalArgumentException(x.toString());
 	}
@@ -247,7 +332,7 @@ implements JavaObjective, java.io.Serializable {
 	 * (non-Javadoc)
 	 * @see net.morilib.lisp.LispNumber#uminus()
 	 */
-	public LispNumber uminus() {
+	public LispDouble uminus() {
 		return new LispDouble(-number);
 	}
 
@@ -256,12 +341,10 @@ implements JavaObjective, java.io.Serializable {
 	 * @see net.morilib.lisp.LispNumber#isEqualTo(net.morilib.lisp.LispNumber)
 	 */
 	public boolean isEqualTo(LispNumber x) {
-		if(x instanceof LispComplex) {
-			return false;
-		} else if(x instanceof LispReal) {
+		if(x instanceof LispReal) {
 			return number == x.getRealDouble();
 		}
-		throw new IllegalArgumentException(x.toString());
+		return false;
 	}
 
 	/*
@@ -460,6 +543,21 @@ implements JavaObjective, java.io.Serializable {
 		return number;
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispNumber#getRealDecimal64()
+	 */
+	@Override
+	public long getRealDecimal64() {
+		return Decimal64.toDecimal(number);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispNumber#getRealDecimal32()
+	 */
+	public int getRealDecimal32() {
+		return Decimal32.toDecimal(number);
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.Datum#getType()
 	 */
@@ -605,6 +703,22 @@ implements JavaObjective, java.io.Serializable {
 		return new Double(number);
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispReal#floor()
+	 */
+	@Override
+	public LispReal floor() {
+		return new LispDouble(Math.floor(number));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispReal#ceil()
+	 */
+	@Override
+	public LispReal ceil() {
+		return new LispDouble(Math.ceil(number));
+	}
+
 	/*
 	 * (non-Javadoc)
 	 * @see java.lang.Object#equals(java.lang.Object)
