@@ -23,7 +23,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.NoSuchElementException;
 
-import net.morilib.lisp.array.LispArray;
+import net.morilib.lisp.array.ILispArray;
 import net.morilib.lisp.array.LispArrayPrototype;
 import net.morilib.lisp.array.LispArrayShape;
 import net.morilib.lisp.array.LispDefaultArray;
@@ -49,7 +49,7 @@ import net.morilib.util.Lists;
  * @author MORIGUCHI, Yuichiro 2009
  */
 public class LispVector extends Datum
-implements LispSequence, LispPurelyMutableCollection, LispArray,
+implements LispSequence, LispPurelyMutableCollection, ILispArray,
 LispArrayPrototype, ILispVector, ILispAddable<LispVector>,
 SRFI95Sequence<LispVector>, ILispIterable, java.io.Serializable {
 
@@ -547,7 +547,7 @@ SRFI95Sequence<LispVector>, ILispIterable, java.io.Serializable {
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArray#isIndexEqualTo(net.morilib.lisp.array.LispArray)
 	 */
-	public boolean isIndexEqualTo(LispArray a) {
+	public boolean isIndexEqualTo(ILispArray a) {
 		return (a.rank() == 1 &&
 				a.startIndex(0) == 0 &&
 				a.endIndex(0) == size());
@@ -556,7 +556,7 @@ SRFI95Sequence<LispVector>, ILispIterable, java.io.Serializable {
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArray#isEqualTo(net.morilib.lisp.array.LispArray)
 	 */
-	public boolean isEqualTo(LispArray a) {
+	public boolean isEqualTo(ILispArray a) {
 		if(!isIndexEqualTo(a)) {
 			return false;
 		} else {
@@ -579,7 +579,7 @@ SRFI95Sequence<LispVector>, ILispIterable, java.io.Serializable {
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArrayPrototype#makeArray(int[])
 	 */
-	public LispArray makeArray(int... is) {
+	public ILispArray makeArray(int... is) {
 		int[] bi = new int[is.length];
 		int[] ei = new int[is.length];
 
