@@ -15,6 +15,7 @@
  */
 package net.morilib.lisp;
 
+import java.util.Comparator;
 import java.util.Iterator;
 import java.util.Map.Entry;
 import java.util.NoSuchElementException;
@@ -24,6 +25,8 @@ import net.morilib.lisp.collection.LispCollection;
 import net.morilib.lisp.collection.LispDictionary;
 import net.morilib.lisp.collection.LispMap;
 import net.morilib.lisp.collection.LispSequence;
+import net.morilib.lisp.iterator.ILispIterable;
+import net.morilib.lisp.iterator.ILispIterator;
 import net.morilib.lisp.sos.LispType;
 import net.morilib.lisp.subr.IsEqual;
 import net.morilib.util.Iterators;
@@ -34,7 +37,9 @@ import net.morilib.util.Iterators;
  *
  * @author MORIGUCHI, Yuichiro 2009
  */
-public final class Nil extends SExpressionDatum {
+public final class Nil extends SExpressionDatum
+implements ConsOrNil, ILispIterable, ILispIterator,
+java.io.Serializable {
 
 	/**
 	 * 
@@ -650,4 +655,95 @@ public final class Nil extends SExpressionDatum {
 		return e.size() == 0;
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.math.algebra.ILispAddable#add(net.morilib.lisp.math.algebra.ILispAddable)
+	 */
+	public SExpression add(SExpression y) {
+		return y;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.math.algebra.ILispMultipliable#mul(net.morilib.lisp.math.algebra.ILispMultipliable)
+	 */
+	public SExpression mul(SExpression y) {
+		return this;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.ConsOrNil#getCar()
+	 */
+	public Datum getCar() {
+		throw new UnsupportedOperationException();
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.ConsOrNil#getCdr()
+	 */
+	public Datum getCdr() {
+		throw new UnsupportedOperationException();
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.sort.SRFI95Sequence#isSorted(java.util.Comparator)
+	 */
+	public boolean isSorted(Comparator<Datum> cmp) {
+		return true;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.sort.SRFI95Sequence#merge(net.morilib.lisp.sort.SRFI95Sequence, java.util.Comparator)
+	 */
+	public ConsOrNil merge(ConsOrNil m, Comparator<Datum> cmp) {
+		return m;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.sort.SRFI95Sequence#mergeS(net.morilib.lisp.sort.SRFI95Sequence, java.util.Comparator)
+	 */
+	public ConsOrNil mergeS(ConsOrNil m, Comparator<Datum> cmp) {
+		return m;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.sort.SRFI95Sequence#sort(java.util.Comparator)
+	 */
+	public ConsOrNil sort(Comparator<Datum> cmp) {
+		return this;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.sort.SRFI95Sequence#sortS(java.util.Comparator)
+	 */
+	public void sortS(Comparator<Datum> cmp) {
+		// do nothing
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.iterator.ILispIterator#isTerminated()
+	 */
+	public boolean isTerminated() {
+		return true;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.iterator.ILispIterator#next()
+	 */
+	public ILispIterator next() {
+		throw new NoSuchElementException();
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.iterator.ILispIterator#getCurrentDatum()
+	 */
+	public Datum getCurrentDatum() {
+		throw new NoSuchElementException();
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.iterator.ILispIterable#lispIterator()
+	 */
+	public ILispIterator lispIterator() {
+		return this;
+	}
+
 }
