@@ -27,6 +27,6 @@ public interface LispArrayPrototype {
 	 * @param is
 	 * @return
 	 */
-	public LispArray makeArray(int... is);
+	public ILispArray makeArray(int... is);
 
 }
