@@ -15,21 +15,36 @@
  */
 package net.morilib.lisp.subr;
 
+import java.util.Arrays;
+
 import net.morilib.lisp.Datum;
 import net.morilib.lisp.Environment;
+import net.morilib.lisp.LispComplex;
 import net.morilib.lisp.LispMessage;
 import net.morilib.lisp.LispNumber;
+import net.morilib.lisp.MultiValues;
 
+/**
+ * 
+ *
+ *
+ * @author MORIGUCHI, Yuichiro 2009
+ */
 public class ImagPart extends UnaryArgs {
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.subr.UnaryArgs#execute(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.LispMessage)
+	 */
 	@Override
 	protected Datum execute(
 			Datum c1a, Environment env, LispMessage mesg) {
-		if(c1a instanceof LispNumber) {
-			return ((LispNumber)c1a).getImag();
+		if(c1a instanceof LispComplex) {
+			return ((LispComplex)c1a).getImag();
+		} else if(c1a instanceof LispNumber) {
+			return MultiValues.newValues(
+					Arrays.asList(((LispNumber)c1a).getImags()));
 		}
 		throw mesg.getError("err.require.number", c1a);
-		//throw new LispException("number required");
 	}
 
 }
